function [ ll , grad , hess ] = ll_rclogit_anal( theta , data , mp )
%LL_RCLOGIT Summary of this function goes here
%   Detailed explanation goes here

global fctCount

fctCount = fctCount+1; 

% utilities
v = data.z*reshape(theta,[mp.D,mp.K])*data.x/mp.lambda; % (NxD * DxK * KxJ) = NxJ
logsum = log(sum(exp(v),2)); 

% CCPs
% Computationally sound
ll = v(data.choiceIdx)' - logsum; 
    
% ### ANALYTIC GRADIENTS ###
ccps = exp(v - logsum(:,ones(1,mp.J))); 
if mp.D>1; 
    error('gradients only work for D==1.'); 
end; 

% Compute gradients 
gradFull = nan(mp.N,mp.K); 
for iK=1:mp.K; 
    dv = data.z(:,ones(1,mp.J)) .* data.x(iK*ones(mp.N,1),:)/mp.lambda; 
    gradFull(:,iK) = -sum(ccps.*dv,2); 
    gradFull(:,iK) = gradFull(:,iK) + dv(data.choiceIdx'); 
end;

% Hessian (if asked for) using BHHH approximation
if nargout == 3; 
    hess = mp.N^-1 * (gradFull'*gradFull); 
end; 
     
grad = -mean(gradFull,1); % already divided by 1/N
ll = - mean(ll); 


end

