%% FINITE DIFFERENCE METHODS --- NUMEDIG SEMINAR

clc; clear all; 

if 0; 
    f  = @(x) exp(x); 
    df = @(x) exp(x); 
    d2f = @(x) exp(x); 
    h = 10.^(-12:0.005:-5)'; 
else 
    f  = @(x) 1 * (x.^3 - 2*x.^2 + 1); 
    df = @(x) 3*x.^2 - 4*x; 
    d2f = @(x) 6*x - 4; 
    h = 10.^( -12:.005:-5 )';
end; 
    
x0 = 1; 
 

fds = nan(numel(h),1); 

for i=1:numel(h); 
    fds(i) = fd(f,x0,h(i)); 
end; 

errs = abs(fds-df(x0)); 

loglog(h,errs,'.k');
title('Absolute error by step size'); 
xlabel('h'); 
ylabel('|FD(x0) - f''(x0)|')


fprintf('Minimum attained for h = %7.5g.\n',h(errs==min(errs))); 

%% Higher order differences
h = []; 
h = 10.^(-9:0.005:-3)'; 
fds = nan(numel(h),1); 

for i=1:numel(h); 
    % use 1e-8 for the FD and h(i) for the second FD
    %fds(i) = hofd(f,x0,[10^-8;h(i)],2); 
    
    % using h(i) as the tolerance both for the inner FD and the second FD
    fds(i) = hofd(f,x0,[h(i)],2); 
end; 

errs = abs(fds-d2f(1)); 

loglog(h,errs,'.k'); 
title('Hessian'); xlabel('h'); ylabel('|f''(x) - f^{*}''(x)|'); 
idx = find(errs==min(errs)); 
fprintf('Minimum attained for h = %7.5g.\n',h(idx(1))); 
if numel(idx)>1; 
    fprintf('(min attained at %i different points)\n',numel(idx)); 
end; 

%% Estimating noice level

nf = 20; 
h = 1e-12; 
fvals = nan(nf,1); 
x0 = 2; 
xvals = x0 + h*(-floor(nf/2):floor(nf/2))'; 
for i=1:nf; 
    fvals(i) = f(xvals(i)); 
%     fvals(i) = hofd(f,xvals(i),1e-5,4); 
end; 

[fnoise,level,inform] = ECNoise(nf,fvals); 

fprintf('ECnoise:     Noise at h=%4.2g = %7.5g.\n',h,fnoise);

% Own attempt at a simple function
%[f_diff,mynoise] = simple_diff(f,x0,h,nf);
%fprintf('simple_diff: Noise at h=%4.2g = %7.5g.\n',h,mynoise); 


%% DDC Noise Estimation

there = '/Users/anders/Dropbox/Skole/PhD/Projects/prj.IRUC/Code/anders'; 
addpath(there);
here = cd(there); 
run_fedor; 
cd(here); 

% IDX_GAMMA_VKT_0    0
% IDX_TRANSCOST_0    1
% IDX_GAMMA_M        2
% IDX_GAMMA_AGE      3
% IDX_TRANSCOST_M    4
% IDX_GAMMA_VKT_AGE1 5
% IDX_GAMMA_VKT_AGE2 6
% IDX_GAMMA_VKT_M    7
p.pnames = { 'transcost_0' }; 
p.pnamesIdx = 1; 
theta0 = p.transcost_0; 

ll = @(theta) mean(ll_ddc(theta,p,spaces,ch,initstates,simdata)); 

%% Estimating DDC noise 
nf = 20; 
h = 1e-15; 
fvals = nan(nf,1); 
x0 = 2; 
xvals = x0 + h*(-floor(nf/2):floor(nf/2))'; 
for i=1:nf; 
    fvals(i) = ll(xvals(i)); 
%     fvals(i) = hofd(f,xvals(i),1e-5,4); 
end; 

[fnoise,level,inform] = ECNoise(nf,fvals); 

fprintf('ECnoise:     Noise at h=%4.2g = %7.5g.\n',h,fnoise); 







